
/* Chrysalide - Outil d'analyse de fichiers binaires
 * fnv1a.c - équivalent Python du fichier "common/fnv1a.c"
 *
 * Copyright (C) 2018-2019 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "fnv1a.h"


#include <pygobject.h>


#include <common/fnv1a.h>


#include "../access.h"
#include "../helpers.h"



/* Détermine l'empreinte FNV1a d'une chaîne de caractères. */
static PyObject *py_fnv1a_hash(PyObject *, PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = NULL car méthode statique.                            *
*                args = arguments fournis lors de l'appel à la fonction.      *
*                                                                             *
*  Description : Détermine l'empreinte FNV1a d'une chaîne de caractères.      *
*                                                                             *
*  Retour      : Numéro de révision.                                          *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_fnv1a_hash(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Instance à retourner        */
    const char *str;                        /* Chaîne à traiter.           */
    int ret;                                /* Bilan de lecture des args.  */
    fnv64_t value;                          /* Empreinte calculée          */

    ret = PyArg_ParseTuple(args, "s", &str);
    if (!ret) return NULL;

    value = fnv_64a_hash(str);

    result = Py_BuildValue("K", (unsigned long long)value);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_fnv1a_type(void)
{
    static PyMethodDef py_fnv1a_methods[] = {

        { "hash", py_fnv1a_hash,
          METH_VARARGS | METH_STATIC,
          "hash(str, /)\n--\n\nCompute the FNV-1a hash from a given string."
        },
        { NULL }

    };

    static PyTypeObject py_fnv1a_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name = "pychrysalide.core.fnv1a",
        .tp_basicsize = sizeof(PyObject),

        .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_IS_ABSTRACT,

        .tp_doc = "Python version for Chrysalide of the Fowler-Noll-Vo hash function.",

        .tp_methods =  py_fnv1a_methods

    };

    return &py_fnv1a_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.core.fnv1a'.           *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_fnv1a_is_registered(void)
{
    PyTypeObject *type;                     /* Type Python pour 'fnv1a'    */
    PyObject *module;                       /* Module à recompléter        */

    type = get_python_fnv1a_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        //type->tp_new = PyType_GenericNew;

        if (PyType_Ready(type) != 0)
            return false;

        module = get_access_to_python_module("pychrysalide.common");

        if (!register_python_module_object(module, type))
            return false;

    }

    return true;

}
